#!/usr/bin/env python

# Script for Face mask with e-paper display - Version 10-May-2020

# from Pimoroni button shim script
import time
import signal
import buttonshim

# import and start display

#from WaveShare-paper script

# -*- coding:utf-8 -*-
import sys
import os

picdir = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), 'pic')
libdir = os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), 'lib')

if os.path.exists(libdir):
    sys.path.append(libdir)

import logging
from waveshare_epd import epd2in9d
import time
from PIL import Image,ImageDraw,ImageFont
import traceback

#Set output log level
logging.basicConfig(level=logging.DEBUG)

'''                  #not required here
font15 = ImageFont.truetype(os.path.join(picdir, 'Font.ttc'), 15)
font24 = ImageFont.truetype(os.path.join(picdir, 'Font.ttc'), 24)
'''

#set BMP list
BMPs = [["Mouth_15.bmp", "Mouth_4.bmp", "Mouth_11.bmp", "Mouth_5.bmp", "Mouth_10.bmp"],
        ["Katze_1.bmp", "Hai_1.bmp", "Hai_4.bmp", "Hund_1.bmp", "Corona_1.bmp"],
        ["TheQueen_2.bmp", "TheQueen_1.bmp", "C_Darwin.bmp", "Vampire_1.bmp", "Mouth_22.bmp"],
        ["Mouth_17.bmp", "Mouth_12.bmp", "Mouth_16.bmp", "Mouth_18.bmp", "Orc_1.bmp"],
        ["White.bmp", "Black.bmp", "Dirt_1.bmp", "Dark_sparks_1.bmp", "Bricks.bmp"]]

#replacement lists
Collection_1 = ["L_Pavarotti_1.bmp", "B_Obama_1.bmp", "M_Ghandi_1.bmp", "D_Lama_1.bmp", "J_Stalin_1.bmp"] # generate lists of your favorite combinations
Collection_2 = ["A_Jolie_1.bmp", "D_Kerr_1.bmp", "C_Sherman_1.bmp", "TheQueen_2.bmp", "Mouth_11.bmp"]
Collection_3 = ["", "","",""]

Collections = [Collection_1, Collection_2] # to be displayed by display_collections function

default_image = "instructables_logo.bmp" # Image to be shown on startup


def display_image(image): # function to write image to e-paper display
    
    Himage = Image.open(os.path.join(picdir, image))
    epd.display(epd.getbuffer(Himage))
    time.sleep(.5)


def display_gallery():     #can be used to list and display all images

    epd = epd2in9d.EPD()
    epd.init()
    epd.Clear(0xFF)
    for i in range (5):
        for j in range (5):
            print (i,"   ",j)
            to_display = BMPs[i][j]            
            print (to_display)
            display_image (to_display)
            time.sleep (.5)             #wait 0.5 seconds before displaying next image       
    


def display_collection():     #can be used to list and display all images

    epd = epd2in9d.EPD()
    epd.init()
    epd.Clear(0xFF)
    for collection in (Collections):
        for i in range (5):
            to_display = collection[i] #pick collection and image to display
            print (to_display)
            display_image (to_display)
            time.sleep (2)             #wait 1 second before next        
        

#Long press detection
@buttonshim.on_hold(buttonshim.BUTTON_A, hold_time=2)
def hold_handler_a (button):
    global button_was_held, panel_flag
    button_was_held = True
    print("Long press button A detected!")
    panel_flag = "panel_1"

@buttonshim.on_hold(buttonshim.BUTTON_B, hold_time=2)
def hold_handler_b (button):
    global button_was_held,panel_flag
    button_was_held = True
    print("Long press button B detected!")
    panel_flag = "panel_2"
    
@buttonshim.on_hold(buttonshim.BUTTON_C, hold_time=2)
def hold_handler_c (button):
    global button_was_held,panel_flag
    button_was_held = True
    print("Long press button C detected!")
    panel_flag = "panel_3"

@buttonshim.on_hold(buttonshim.BUTTON_D, hold_time=2)
def hold_handler_d (button):
    global button_was_held,panel_flag
    button_was_held = True
    print("Long press button D detected!")
    panel_flag = "panel_4"

@buttonshim.on_hold(buttonshim.BUTTON_E, hold_time=2)
def hold_handler_e (button):
    global button_was_held,panel_flag
    button_was_held = True
    print("Long press button E detected!")
    panel_flag = "panel_5"

#short press detection
@buttonshim.on_release(buttonshim.BUTTON_A)  
def button_a(button, pressed):
    global button_was_pressed, button_flag
    button_was_pressed = True 
    print("Short press button A detected!")
    button_flag = "button_1"
       
@buttonshim.on_release(buttonshim.BUTTON_B)
def button_b(button, pressed):
    global button_was_pressed, button_flag
    button_was_pressed = True
    print("Short press button B detected!")
    button_flag = "button_2"
        
@buttonshim.on_release(buttonshim.BUTTON_C)
def button_c(button, pressed):
    global button_was_pressed, button_flag
    button_was_pressed = True
    print("Short press button C detected!")
    button_flag = "button_3"
 
@buttonshim.on_release(buttonshim.BUTTON_D)
def button_d(button,pressed):
    global button_was_pressed, button_flag
    button_was_pressed = True
    print("Short press button D detected!")
    button_flag = "button_4"

@buttonshim.on_release(buttonshim.BUTTON_E)
def button_e(button,pressed):  
    global button_was_pressed, button_flag
    button_was_pressed = True
    print("Short press button E detected!")
    button_flag = "button_5"

#set base state of flags and working variables
button_flag = "null"
panel_flag = "null"
image = 0 # image indicator   
panel = 0 #panel indicator: sets panel 1 as base panel 
button_was_held = False
button_was_pressed = False
to_display="xxx.bmp"

try:
    logging.info("epd2in9d w/ button shim")
    
    epd = epd2in9d.EPD()
    logging.info("init and clear")
    epd.init()
    epd.Clear(0xFF)  # write white
    display_image (default_image) #display default image
    time.sleep(2)
    
    display_collection()
    #display_gallery()
    
    print ("""
           Button SHIM: control-main.py, adapted for 2in9d display.
           Light up the LED a different color of the rainbow with each button pressed
           and display a certain image on the display.
           """)

    while True:
        
        time.sleep(.5) # pause between check circles         
        
        if (button_was_held == True):
            #print (panel_flag, button_flag)
            
            if panel_flag == "panel_1":
                panel = 0
                  
            elif panel_flag == "panel_2":
                panel = 1
                  
            elif panel_flag == "panel_3":
                panel = 2
                  
            elif panel_flag == "panel_4":
                panel = 3
                  
            elif panel_flag == "panel_5":
                panel = 4

            panel_flag = "null"
            button_was_held = False
            buttonshim.set_pixel(0x94, 0x94, 0x94) #white, as panel change indicator
            button_was_pressed=False
            #logging.info ("clear display") # optional
            #epd.Clear(0xFF)  # write white - optional clear display step              
 
 
        elif (button_was_pressed == True):
            
            if button_flag == "button_1":
                   buttonshim.set_pixel(0x94, 0x00, 0xd3) #violet
                   image = 0
                    
            elif button_flag == "button_2":
                   buttonshim.set_pixel(0x00, 0x00, 0xff) #blue
                   image = 1
                  
            elif button_flag == "button_3":    
                   buttonshim.set_pixel(0x00, 0xff, 0x00) #green
                   image = 2
                  
            elif button_flag == "button_4":       
                   buttonshim.set_pixel(0xff, 0xff, 0x00)  #yellow
                   image = 3

            elif button_flag == "button_5":   
                   buttonshim.set_pixel(0xff, 0x00, 0x00) #red
                   image = 4
               
            to_display = BMPs[panel][image]
            print (button_flag,": ",to_display)
            # epd.Clear(0xFF)                           
            display_image(to_display)
            button_was_pressed=False


    logging.info("Clear...")
    epd.Clear(0xFF)
    
    logging.info("Goto Sleep...")
    epd.sleep()
        
except IOError as e:
    logging.info(e)
    
except KeyboardInterrupt:    
    logging.info("ctrl + c:")
    epd2in9d.epdconfig.module_exit()
    exit()
